<?php

namespace Misaz\WYSIWYM;


use Misaz\WYSIWYM\Convertors\BoldConvertor;
use Misaz\WYSIWYM\Convertors\HyperlinkConvertor;
use Misaz\WYSIWYM\Convertors\ImageConvertor;
use Misaz\WYSIWYM\Convertors\ItalicConvertor;
use Misaz\WYSIWYM\Convertors\ListConvertor;
use Misaz\WYSIWYM\Convertors\ParagraphConvertor;
use Misaz\WYSIWYM\Convertors\Title1Convertor;
use Misaz\WYSIWYM\Convertors\Title2Convertor;

/**
 * Class WYSIWYM WYSIWYM convertor
 * @package Misaz\WYSIWYM
 */
class WYSIWYM {

	/**
	 * @var string input content
	 */
	private $content;

	public function __construct($content) {
		$this->content = str_replace("\r", "", $content);
		$this->convert();
	}

	/**
	 * Convert input. In input will be passed converted output
	 * @param null $convertors covnertors used to converting input. Null uses default convertors
	 */
	private function convert($convertors = null) {
		if (!$convertors) { // loads default
			$convertors = self::getDefaultConvertors();
		}

		foreach ($convertors as $convertor) {
			/** @var IConvertor $convertor */
			$this->content = $convertor->convert($this->content);
		}
		$xss = new XssGuard();
		$whitelist = $convertors[count($convertors) - 1]->avaibleTags;
		$this->content = $xss->safe($this->content, $whitelist);
	}

	/**
	 *
	 * Returns converted HTML
	 * @return string HTML content
	 */
	public function getHTML() {
		return $this->content;
	}

	/**
	 * Returns collection with default convertors and intialize avaible tags.
	 * @return array convertors
	 */
	public static function getDefaultConvertors() {
		$paragraph = new ParagraphConvertor(); // paragraph need list of avaible tags
		$convertosrs = array(
			new ImageConvertor(),
			new ListConvertor(),
			new BoldConvertor(),
			new ItalicConvertor(),
			new Title1Convertor(),
			new Title2Convertor(),
			new HyperlinkConvertor(),
			$paragraph
		);

		foreach ($convertosrs as $c) {
			/** @var IConvertor $c */
			$paragraph->avaibleTags = array_merge($paragraph->avaibleTags, $c->avaibleTags());
		}
		return $convertosrs;
	}

}