<?php

namespace Misaz\WYSIWYM;

use DOMDocument;
use DOMElement;
use DOMNode;

/**
 * Protect HTML input to XSS attack
 * Class XssGuard
 * @package Misaz\WYSIWYM
 */
class XssGuard {

	/**
	 * Protect input from XSS attack
	 * @param string $input
	 * @param array $tagWhitelist
	 * @return string
	 */
	public function safe($input, $tagWhitelist) {
		$tagWhitelist = array_merge($tagWhitelist, array("html", "body")); // prepare whitelist

		$xml = new DOMDocument(1.0, "utf8"); // load XML document
		$xml->loadHTML('<?xml encoding="utf-8" ?>' . $input); // set UTF-8 and load HTML

		$this->protectElement($xml->documentElement, $tagWhitelist); // call root recursive call

		$b = $xml->getElementsByTagName("body");

		$output = $xml->saveHTML($b->item(0)); // export body element
		return substr($output, 6, strlen($output) - 13); // returns innerHTML of body
	}

	/**
	 * Protect element from XSS and call it recursivelo on his nodes
	 * @param DOMElement $element Tested element
	 * @param array $tagWhitelist whitelisted elements
	 */
	private function protectElement($element, $tagWhitelist) {
		$childs = $element->childNodes; // get childs
		foreach ($childs as $child) { // recursively call this same on childs
			/** @var $child DOMNode */
			if ($child->nodeType == XML_ELEMENT_NODE) { // only on elemenets
				$this->protectElement($child, $tagWhitelist);
			}
		}

		// check this tag
		$tag = $element->localName;
		if (array_search($tag, $tagWhitelist) === false) { // potential issue !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
			// change the tag to <span>
			$oldChilds = $element->childNodes;
			$newElement = $element->ownerDocument->createElement("span");
			foreach ($oldChilds as $c) { // copy content
				$newElement->appendChild($c);
			}
			$element->parentNode->replaceChild($newElement, $element);
		}
	}
}